#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt
import json, codecs
from pathlib import Path
from scipy.optimize import curve_fit
import scipy
import glob
import os



'''
Write here the path to the folder where this script is stored 
'''
scriptpath = 'YOURPATH'

def function_Rabi(t, A, TR, f_r, phase, C):                              # first argument is the independent variable
    S21 = A * (np.exp(-(t/TR))) * (np.sin(2*np.pi*f_r*t+phase)) + C
    return S21
 
 
 
def fit_Rabi(duration_pump_array, rabi_array):
    x = duration_pump_array*1e-9
    y = rabi_array
    p0 = np.array([4e-4, 200e-9, 10e6, -np.pi/2, 4e-4])                  # initial guesses
    p_min = [1e-4, 20e-9, 2e6,  -np.pi, 1e-4]
    p_max = [12e-4, 600e-9, 100e6, np.pi, 10e-4]  
    popt, pcov = curve_fit(function_Rabi, x, y, p0, bounds=(p_min,p_max))
    return popt
 

#%%
'''Figure 2a: Q1'''


reference_arrays_magnitude =np.load(scriptpath + 'a/Q1/reference_array_total_magnitude.npy')
measurement_arrays_magnitude = np.load(scriptpath + 'a/Q1/measurement_array_total_magnitude.npy')
reference_arrays_phase =np.load(scriptpath + 'a/Q1/reference_array_total_phase.npy')
measurement_arrays_phase = np.load(scriptpath + 'a/Q1/measurement_array_total_phase.npy')
reference_arrays_I =np.load(scriptpath + 'a/Q1/reference_array_total_I.npy')
measurement_arrays_I = np.load(scriptpath + 'a/Q1/measurement_array_total_I.npy')
reference_arrays_Q =np.load(scriptpath + 'a/Q1/reference_array_total_Q.npy')
measurement_arrays_Q = np.load(scriptpath + 'a/Q1/measurement_array_total_Q.npy')



'Choose which signal to use, in the paper the magnitude is used'
signal = 'magnitude'

if signal == 'magnitude': 
    reference_arrays = reference_arrays_magnitude
    measurement_arrays = measurement_arrays_magnitude
elif signal == 'phase':
    reference_arrays = reference_arrays_phase
    measurement_arrays = measurement_arrays_phase
elif signal == 'I':
    reference_arrays = reference_arrays_I
    measurement_arrays = measurement_arrays_I
elif signal == 'Q':
    reference_arrays = reference_arrays_Q
    measurement_arrays = measurement_arrays_Q
    
''''Now the reference_arrays and measurement_arrays contain all the readout traces of the reference and measurement respectively
of all plotted datapoints 
'''
data_array = []
data_arrays = []
for i in range (1): #If taking a train average, this number will be >1, in this case we do not
    for k in range (51): #In this measurement we acquire 51 datapoints
        difference_array_sum = [0]
        difference_array = np.abs((reference_arrays[i][k]-measurement_arrays[i][k]))
        for element in difference_array: #Here the sum is taken over the difference between the reference and measurement traces for the 12 consecutive segments
            difference_array_sum = difference_array_sum+element
        data_array.append(difference_array_sum[0])
    data_arrays.append(data_array)
    data_array = []
        
    
fig = plt.figure(figsize = (8,6))    
sweep_array = np.linspace(1,201,51)
averaged_data_array = np.average(data_arrays, axis = 0) # here the average is taken over the 10 traces
plt.plot(sweep_array, averaged_data_array, 'o') #this is the data plotted in the paper 
plt.xlabel('Burst time (ns)')
plt.ylabel('\u0394 |S21mag| (arb. un.)')
popt = fit_Rabi(sweep_array, averaged_data_array )
plt.plot(sweep_array, function_Rabi(sweep_array*1e-9, *popt))
#%%
'''Figure 2b: Q2'''


reference_arrays_magnitude =np.load(scriptpath + 'b/Q2/reference_array_total_magnitude.npy')
measurement_arrays_magnitude = np.load(scriptpath + 'b/Q2/measurement_array_total_magnitude.npy')
reference_arrays_phase =np.load(scriptpath + 'b/Q2/reference_array_total_phase.npy')
measurement_arrays_phase = np.load(scriptpath + 'b/Q2/measurement_array_total_phase.npy')
reference_arrays_I =np.load(scriptpath + 'b/Q2/reference_array_total_I.npy')
measurement_arrays_I = np.load(scriptpath + 'b/Q2/measurement_array_total_I.npy')
reference_arrays_Q =np.load(scriptpath + 'b/Q2/reference_array_total_Q.npy')
measurement_arrays_Q = np.load(scriptpath + 'b/Q2/measurement_array_total_Q.npy')



'Choose which signal to use, in the paper the magnitude is used'
signal = 'magnitude'

if signal == 'magnitude': 
    reference_arrays = reference_arrays_magnitude
    measurement_arrays = measurement_arrays_magnitude
elif signal == 'phase':
    reference_arrays = reference_arrays_phase
    measurement_arrays = measurement_arrays_phase
elif signal == 'I':
    reference_arrays = reference_arrays_I
    measurement_arrays = measurement_arrays_I
elif signal == 'Q':
    reference_arrays = reference_arrays_Q
    measurement_arrays = measurement_arrays_Q
    
''''Now the reference_arrays and measurement_arrays contain all the readout traces of the reference and measurement respectively
of all plotted datapoints 
'''
data_array = []
data_arrays = []
for i in range (1): #If taking a train average, this number will be >1, in this case we do not
    for k in range (51): #In this measurement we acquire 51 datapoints
        difference_array_sum = [0]
        difference_array = np.abs((reference_arrays[i][k]-measurement_arrays[i][k]))
        for element in difference_array: #Here the sum is taken over the difference between the reference and measurement traces for the 12 consecutive segments
            difference_array_sum = difference_array_sum+element
        data_array.append(difference_array_sum[0])
    data_arrays.append(data_array)
    data_array = []
        
    
fig = plt.figure(figsize = (8,6))    
sweep_array = np.linspace(1,201,51)
averaged_data_array = np.average(data_arrays, axis = 0) # here the average is taken over the 10 traces
plt.plot(sweep_array, averaged_data_array, 'o') #this is the data plotted in the paper 
plt.xlabel('Burst time (ns)')
plt.ylabel('\u0394 |S21mag| (arb. un.)')
popt = fit_Rabi(sweep_array, averaged_data_array )
plt.plot(sweep_array, function_Rabi(sweep_array*1e-9, *popt))
    